package gz.cyberbrain.cyberhotel.controller;

import gz.cyberbrain.cyberhotel.model.board.to.BoardTo;
import gz.cyberbrain.cyberhotel.model.client.to.AgencyTo;
import gz.cyberbrain.cyberhotel.model.client.to.ClientTo;
import gz.cyberbrain.cyberhotel.model.client.to.CompanyTo;
import gz.cyberbrain.cyberhotel.model.client.to.ParticularTo;
import gz.cyberbrain.cyberhotel.model.configurationfacade.delegate.ConfigurationFacadeDelegate;
import gz.cyberbrain.cyberhotel.model.configurationfacade.exceptions.UnaceptableBoardException;
import gz.cyberbrain.cyberhotel.model.configurationfacade.exceptions.UnaceptablePriceTypeException;
import gz.cyberbrain.cyberhotel.model.pricetype.to.PriceTypeTo;
import gz.cyberbrain.cyberhotel.model.request.to.RequestTo;
import gz.cyberbrain.cyberhotel.model.reservation.to.ReservationTo;
import gz.cyberbrain.cyberhotel.model.reservationfacade.delegate.ReservationFacadeDelegate;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.RegisteredReservationException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.UnaceptableAgencyException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.UnaceptableCompanyException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.UnaceptableParticularException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.UnaceptablePayerException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.UnaceptableRoomException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.UnavailableRoomException;
import gz.cyberbrain.cyberhotel.model.room.to.RoomTo;
import gz.cyberbrain.cyberhotel.model.spring.facade.SpringFacadeFactory;
import gz.cyberbrain.cyberhotel.model.util.exceptions.InstanceNotFoundException;
import gz.cyberbrain.cyberhotel.model.util.exceptions.InternalErrorException;
import gz.cyberbrain.cyberhotel.model.util.exceptions.InvalidIntervalException;
import gz.cyberbrain.cyberhotel.view.swing.util.gui.TextItem;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Vector;

public class ReservationController {

	private static ConfigurationFacadeDelegate config = SpringFacadeFactory.getConfigurationFacadeDelegate();
	private static ReservationFacadeDelegate delegate = SpringFacadeFactory.getReservationFacadeDelegate();
	private static HashMap<String, Object> attributes = new HashMap<String, Object>();

	private ReservationController() {}

	public static Vector reservationList() throws InternalErrorException {				
		Vector<TextItem> items = new Vector<TextItem>();

		Collection reservations = delegate.showUnregisteredReservations();

		Iterator iter = reservations.iterator();

		ReservationTo reservation;

		for (int i=0; iter.hasNext(); i++) {

			reservation = (ReservationTo) iter.next();
			items.add(new TextItem(reservation.getReservationId(), reservation.getReservatorName()));

		}

		reservations = delegate.showCancelledReservations();

		iter = reservations.iterator();

		for (int i=0; iter.hasNext(); i++) {

			reservation = (ReservationTo) iter.next();
			items.add(new TextItem(reservation.getReservationId(), reservation.getReservatorName()));

		}

		return items;
	}

	public static Vector unregisteredReservations() throws InternalErrorException {				
		Vector<TextItem> items = new Vector<TextItem>();

		Collection reservations = delegate.showUnregisteredReservations();

		Iterator iter = reservations.iterator();

		ReservationTo reservation;

		for (int i=0; iter.hasNext(); i++) {

			reservation = (ReservationTo) iter.next();
			items.add(new TextItem(reservation.getReservationId(), reservation.getReservatorName()));

		}

		return items;
	}

	
	public static Serializable createReservation(String reservatorName, String reservatorCif, String reservatorAccount, 
			String reservatorCard, String reservatorPhone, String reservationStatus, Calendar incomingDate, 
			Calendar expectedOutgoingDate, Calendar trueOutgoingDate, Long payer, Long agency, Long company, 
			String notes, boolean byHours, Vector requests)
	throws UnaceptableParticularException, UnaceptableBoardException, UnaceptablePriceTypeException, 
	UnaceptableAgencyException, UnaceptablePayerException, UnaceptableCompanyException, UnaceptableRoomException, 
	UnavailableRoomException, InvalidIntervalException, InternalErrorException {

		Collection <RequestTo> requestsTo = new ArrayList <RequestTo> ();

		for (int i = 0; i < requests.size(); i++)
			requestsTo.add(new RequestTo(null, (Long) ((HashMap) requests.elementAt(i)).get("particular"),(String) ((HashMap) requests.elementAt(i)).get("room"), (String) ((HashMap) requests.elementAt(i)).get("board"),(String) ((HashMap) requests.elementAt(i)).get("priceType")));

		return delegate.makeAReservation(new ReservationTo(null, reservatorName, reservatorCif, reservatorAccount, 
				reservatorCard, reservatorPhone, reservationStatus, incomingDate, expectedOutgoingDate, 
				trueOutgoingDate, payer, agency, company, notes, byHours), requestsTo).getReservationId();

	}

	public static void updateReservation(Long reservationId, String reservatorName, String reservatorCif, String reservatorAccount, 
			String reservatorCard, String reservatorPhone, String reservationStatus, Calendar incomingDate, 
			Calendar expectedOutgoingDate, Calendar trueOutgoingDate, Long payer, Long agency, Long company, 
			String notes, boolean byHours, Vector requests) 
	throws UnaceptableParticularException, UnaceptableBoardException, UnaceptablePriceTypeException, 
	UnaceptableAgencyException, UnaceptablePayerException, UnaceptableCompanyException, UnaceptableRoomException, 
	UnavailableRoomException, InvalidIntervalException, InstanceNotFoundException, InternalErrorException {

		Collection <RequestTo> requestsTo = new ArrayList <RequestTo> ();

		for (int i = 0; i < requests.size(); i++) 
			requestsTo.add(new RequestTo(null, (Long) ((HashMap) requests.elementAt(i)).get("particular"),(String) ((HashMap) requests.elementAt(i)).get("room"), (String) ((HashMap) requests.elementAt(i)).get("board"),(String) ((HashMap) requests.elementAt(i)).get("priceType")));			

		delegate.updateAReservation(new ReservationTo(reservationId, reservatorName, reservatorCif, reservatorAccount, 
				reservatorCard, reservatorPhone, reservationStatus, incomingDate, expectedOutgoingDate, 
				trueOutgoingDate, payer, agency, company, notes, byHours), requestsTo);

	}

	public static void deleteReservation(Long reservationId) throws RegisteredReservationException, InstanceNotFoundException, InternalErrorException {

		ReservationTo reservation = delegate.findAReservation(reservationId);
		delegate.removeAReservation(reservation);

	}

	public static Collection reservationRequests(Long reservationId) throws InternalErrorException {
		return delegate.reservationRequests(reservationId);
	}

	public static HashMap findReservation(Long reservationId) throws InstanceNotFoundException, InternalErrorException {

		ReservationTo reservation = delegate.findAReservation(reservationId);

		attributes.put("id", reservationId);
		attributes.put("reservatorName", reservation.getReservatorName());
		attributes.put("reservatorCif", reservation.getReservatorCif());
		attributes.put("reservatorPhone", reservation.getReservatorPhone());		
		attributes.put("reservatorAccount", reservation.getReservatorAccount());
		attributes.put("reservatorCard", reservation.getReservatorCard());
		attributes.put("reservationStatus", reservation.getReservationStatus());
		attributes.put("incomingDate", reservation.getIncomingDate());
		attributes.put("expectedOutgoingDate", reservation.getExpectedOutgoingDate());
		attributes.put("trueOutgoingDate", reservation.getTrueOutgoingDate());

		ClientTo payer = delegate.findAPayer(reservation.getPayer());
		attributes.put("payer", new TextItem(payer.getClientId(), payer.getReferenceName()));

		if (reservation.getAgency() != null) {
			AgencyTo agency = delegate.findAnAgency(reservation.getAgency());
			attributes.put("agency", new TextItem(agency.getClientId(), agency.getName()));
		} else attributes.put("agency", new TextItem(new Long(0), ""));


		if (reservation.getCompany() != null) {
			CompanyTo company = delegate.findACompany(reservation.getCompany());
			attributes.put("company", new TextItem(company.getClientId(), company.getName()));
		} else attributes.put("company", new TextItem(new Long(0), ""));

		//Only for individual reservations:

		Vector request = findRequests(reservationId);

		if (request.size() == 1) {

			attributes.put("particular", ((HashMap) request.elementAt(0)).get("particular"));
			attributes.put("room", ((HashMap) request.elementAt(0)).get("room"));
			attributes.put("board", ((HashMap) request.elementAt(0)).get("board"));
			attributes.put("priceType", ((HashMap) request.elementAt(0)).get("priceType"));

		}

		attributes.put("notes", reservation.getNotes());

		return attributes;

	}

	public static Vector findRequests(Long reservationId) {		
		
		Collection requests;
		Vector <HashMap> requestsVector = new Vector <HashMap> ();
		
		try {
			
		requests = delegate.reservationRequests(reservationId);
	
		if (requests.size() == 0) throw new InternalErrorException(new Exception("Incorrect number of requests."));

		Iterator iter = requests.iterator();

		for (int i=0; iter.hasNext(); i++) {

			RequestTo request = (RequestTo) iter.next();
			HashMap <String, Object> currentRequest = new HashMap <String, Object> ();

			ParticularTo particular = delegate.findAParticular(request.getParticular());
			currentRequest.put("particular", new TextItem(particular.getClientId(), particular.getReferenceName()));

			RoomTo room = config.findRoom(request.getRoom());		
			currentRequest.put("room", new TextItem(room.getRoomCode(), room.getNumber()));

			BoardTo board = config.findBoard(request.getBoard());
			currentRequest.put("board", new TextItem(board.getBoardCode(), board.getDescription()));

			PriceTypeTo priceType = config.findPriceType(request.getPriceType());
			currentRequest.put("priceType", new TextItem(priceType.getPriceTypeCode(), priceType.getDescription()));

			requestsVector.add(currentRequest);

		}

		} catch (InstanceNotFoundException e) {
			e.printStackTrace();
		} catch (InternalErrorException e) {
			e.printStackTrace();
		} 
		
		return requestsVector;

	}

}
